#! /usr/bin/perl -w

# 
# -------------------------------------------------------------------
#                                   X-BONE
#
#                          http://www.isi.edu/xbone
#                USC Information Sciences Institute (USC/ISI)
#                   Marina del Rey, California 90292, USA
#                          Copyright (c) 1998-2005
# 
# -------------------------------------------------------------------
#
# Copyright (c) 1998-2005 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation in source and binary forms for non-commercial
# purposes and without fee is hereby granted, provided that the above
# copyright notice appear in all copies and that both the copyright
# notice and this permission notice appear in supporting
# documentation, and that any documentation, advertising materials,
# and other materials related to such distribution and use acknowledge
# that the software was developed by the University of Southern
# California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived
# from this software without specific prior written permission.
# 
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT
# THE SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS
# PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so
# noted when applicable.
#
# -------------------------------------------------------------------
#
# Effort partly sponsored by the Defense Advanced Research Projects
# Agency (DARPA) and Air Force Research Laboratory, Air Force Materiel
# Command, USAF, under agreement numbers F30602-98-1-0200 (X-Bone) and
# F30602-01-2-0529 (DynaBone). The views and conclusions contained
# herein are those of the authors and should not be interpreted as
# necessarily representing the official policies or endorsements,
# either expressed or implied, of the Defense Advanced Research
# Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
#
# This work was partly supported by the NSF STI-XTEND (ANI-0230789)
# and NETFS (ANI-0129689) projects. Any opinions, findings, and
# conclusions or recommendations expressed in this material are those
# of the authors and do not necessarily reflect the views of the
# National Science Foundation.
#
# ----------------------------------------------------------------------------
#  $RCSfile: dump.pl,v $
#
# $Revision: 1.5 $
#   $Author: pingali $
#     $Date: 2005/03/31 07:03:45 $
#    $State: Exp $
# ----------------------------------------------------------------------------
# $Log: dump.pl,v $
# Revision 1.5  2005/03/31 07:03:45  pingali
# Updated copyright
#
# Revision 1.4  2004/01/22 23:51:17  rzhou
# add copyright
#
# Revision 1.3  2002/12/10 23:12:42  larse
# Merge Sunhee's PRM stuff from the BRANCH_FTN_PI_JUL2002 branch.
#
# Revision 1.2.2.9  2002/09/03 22:04:43  sunhee
# attacked list of innerlay
#
# Revision 1.2.2.8  2002/07/25 00:09:57  sunhee
# tcpdump option change to -s 65535 from 500, because there was tcp packet frame with -s 500 and doesn't work properly.
# But with this value, there still exists udp packet frame.
#
# Revision 1.2.2.7  2002/07/24 23:42:37  sunhee
# Add "R" option for Inbound UDP packet fragment in printing
#
# Revision 1.2.2.6  2002/07/24 23:17:29  sunhee
# Add "r" option for UDP packet fragment in printing
#
# Revision 1.2.2.5  2002/07/23 23:52:20  sunhee
#  tcpdump option change -s 500 from 65535
#
# Revision 1.2.2.4  2002/07/23 23:38:31  sunhee
# tcpdump option change -s 65535 from 200
#
# Revision 1.2.2.3  2002/07/22 20:11:59  sunhee
# return original
#
# Revision 1.2.2.2  2002/07/22 19:41:13  sunhee
# test1
#
# Revision 1.2.2.1  2002/07/05 21:58:11  yushunwa
# Expanded for Dynabone demo (FTNPI-July-2002). Show a single-character symbol
# at the n-th column corresponding to a packet traversing n-th innerlay.
#
# Revision 1.2  2002/06/04 22:41:35  yushunwa
# Added debug mode to dump pkt contents.
#
# Revision 1.1  2002/01/14 01:42:16  yushunwa
# Script to grep tcpdump output to show pkgs through different innerlays.
#
# Revision 1.1  2002/01/12 04:17:01  yushunwa
# Script to combine innerlay barp tables into a single outerlay barp table.
#
# ----------------------------------------------------------------------------
#
# A simple script to take a barp table and its corresponding outerlay address
# to perform tcpdump to show packets going through different innerlays.
#
# Usage: dump.pl -barp <barpfile> -outerlay <outerlay_IP>
#

use strict;
use Getopt::Long;
use Data::Dumper;
use FileHandle;
use FindBin;

# clear path for taint mode
$ENV{PATH} = "";


# Command line options
#
my $barpfile;
my $outerlay;
my $interface;
my %barp;
my @in;
my $debug=0;
my $dump_all=0;    # Warning: Show ALL packets.
#my $interface = "xl0";

sub usage ($) {
  my $msg = shift;

  print "ERROR:   $msg\n" if defined $msg and $msg ne "";
  print <<EOB

USAGE:   $FindBin::Script -outerlay <IP> -barp <file>

OPTIONS: -outerlay    IP address of the outerlay (A.B.C.D)

         -barp        file name of the BGP/ARP table (Data::Dumper format)

	 -d           debug mode, shows every packets

EOB
;
  exit;
}


if (GetOptions("barp=s"        => \$barpfile,
               "outerlay=s"    => \$outerlay,
	       "interface=s"   => \$interface,
	       "d=i"           => \$debug) == 0) { usage "";}

#print "Debug= $debug\n";
#exit;

if ($outerlay !~ /^((\d{1,3}\.){3}\d{1,3})$/)
{ usage "Invalid outerlay $outerlay"; } 

# read in the BARP table
open FH, $barpfile or die "cannot open $barpfile: $!";
my $barptext;
while (<FH>) { chomp; $barptext .= $_; }
close FH or die "cannot close $barpfile: $!";

# untaint BARP file contents and evaluate the contents
# XXX security hole!
if ($barptext =~ /(.*)/) { $barptext = $1; }
else { usage "Invalid BARP table contents"; }
eval $barptext or die "Error evaluting BARP table: $@";

# sanity check: BARP table must contain local outerlay IP address

usage "BARP table does not contain outerlay address $outerlay"
  unless exists $barp{$outerlay};

my $count=0;
for my $n (sort (keys %{$barp{$outerlay}})){
  $count++;
  push @in, $n;
}

print "   # of Innerlays = $count\n";
#print @in, "\n";
print "#### innerlay 0 1 2\n";
print "$barp{$outerlay}{$in[0]} $barp{$outerlay}{$in[1]} $barp{$outerlay}{$in[2]}\n"; 

#$interface = $ARGV[0];
print "Interface on this machine : $interface\n";

my $tcpdump = "/usr/sbin/tcpdump";
#my $TCPDUMP_OPT = "-lnNv -s 65535 -i xl0";
my $TCPDUMP_OPT = "-lnNv -s 65535 -i $interface";
my $dump = "$tcpdump $TCPDUMP_OPT |";

open TCPDUMP, $dump or die "Couldn't open $tcpdump: $!";
while (my $l = <TCPDUMP>){
  my $report = "";
  my $symbol="";
  if ($dump_all) { print "LINE: $l"; }
  #---------------------------------------------------------
  # Check which innerlay the packet goes through, display a
  # single-character symbol at the corresponding column:
  #   S|s = TCP SYN
  #   F|f = TCP FIN
  #   T|t = TCP data
  #   I|i = ICMP
  #   U|u = UDP data
  #   O|o = Other
  #   Uppercase = Inbound
  #   Lowercase = Outbound
  #---------------------------------------------------------
  for(my $i = 0; $i < $count; $i++){
    if($l =~ /$barp{$outerlay}{$in[$i]}(.*)/g){
      my $n = $1;
      #print "n =$n\n";
      if($n =~ /S(\S)? \[tcp/){
	$symbol = "s";
	$report .= "TCP  SYN  ";
      }elsif($n =~ /F(\S)? \[tcp/){
	$symbol = "f";
	$report .= "TCP  FIN  ";
      }elsif($n =~ /(\.|P) \[tcp/){
	$symbol = "t";
	$report .= "TCP ";
      }elsif($n =~ /icmp: ((\w|\s|\.)*)/){
	$symbol = "i";
	$report .= "ICMP $1";
      }elsif($n =~ /udp/){
	$symbol = "u";
	$report .= "UDP";
      }elsif($n =~ /\> (\w|\.|)*\: \(frag/){  #for Outbound UDP traffic only
	$symbol = "r";
	$report .= "Fragment";
      }elsif($n =~ /\: \(frag/){              #for Inbound UDP traffic only
	$symbol = "R";
	$report .= "Fragment";
      }elsif($n =~ /P (\d+:\d+\(\d+\))/){
	$symbol = "t";
	$report .= "TCP  ";
      }else{
	$symbol = "o";
	$report .= "Other \n$l"
      }
      if($n =~ /^:/) { # inbound
	$symbol = uc $symbol;
	$report = "inbound:  ".$report;
      }else{           # outbound
	$symbol = lc $symbol;
	$report = "outbound: ".$report;
      }
      print " "x $i . $symbol. "\n";
      #print "i= $i\n";
      if($debug){ print "$report\n$l";}
      last;
    }
  }
}
